/*
 * Main.java
 *
 * Created on 13. listopad 2006, 14:59
 *
 * @author Jiri Kremser
 * @version 1.0
 * @windowtitle class Main
 */


package dam;
    import java.util.Random;


/**
 * class Main
 */
public class Main {
    
    private static final int chessBoardAmount = 5;
    private static final int[] defaultBoard =  {7,4,2,5,8,6,3,1};    
    private static int[] diagonalBoard = new int[99];
    private static int[][] randomBoard = new int[Main.chessBoardAmount][99];

 /**
 * fillArray(boolean)
 *
 * @param which true fills randomBoard, false fills diagonalArray
 */
    public static void fillArray(boolean which) {
        int first, hShift = 2;
        Random ran = new Random();
        if (which) {        
            for (int i = 0; i < Main.chessBoardAmount; i++) {
                
                first = ran.nextInt(Node.N);
                for (int j = 0; j < Node.N; j++) {

                        if (ran.nextInt(1) == 1) {
                            hShift = (- hShift);
                        }
                        else{
                            hShift += ran.nextInt(Node.N);
                        }
                    randomBoard[i][j] = ((first + hShift) % Node.N) + 1;
                    hShift = hShift + ((hShift < 0) ? -2 : 2);
                }
            }
        }
        else {
            for (int i = 0; i < Node.N; i++) {
                diagonalBoard[i] = (Node.N - i);
            }
        }
    }
    
    
    /**
     * problemSolution(Node)
     *
     * @param first node (default position)
     */
    public static void problemSolution(Node first) {
        final int MAX_ITER = 100;
        int i = 0;
        Node akt = first;
        while (true) {
            System.out.println( (i == 0) ? "\nSTART:" : "STEP " + i + ":" );
            System.out.println(akt);
            if ( (i > MAX_ITER) || (akt.isEnd()))  {
                System.out.println("\n*******************************************");
                System.out.println("  Solution of the problem has" + ((akt.isEnd())?"":"n't") + " been found");
                System.out.println("             in " + i + " steps");
                System.out.println("*******************************************\n");
                break;
            }
            akt = new Node(akt.selectBest());
            i++;
        }
        System.out.println("\n\n\n\n");
    }

    
    /**
     * main()
     *
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        Node first;
        fillArray(true);
        if (args.length > 2) {
            System.out.println("SYNTAX:\n   program_name [chessBoard] [dimension]" +
                               "\n  chessBoard:\n     -r   random chessboards\n" +
                               "     -d   diagonal chessboard\n\n" +
                               "  dimension:\n     dimension of the chessboard (5 - 99)");
            return;
        }        
        if (args.length >= 1) {
            if (args.length == 2) {
                Node.N = Integer.parseInt(args[1]);
            }
            else {
                Node.N = 8; //default dimension
            }
            diagonalBoard = new int[Node.N];
            randomBoard = new int[Main.chessBoardAmount][Node.N];                        
            
            if (args[0].equals("-r")) {
                Random ran = new Random();
                    System.out.println("\nRANDOM CHESSBOARDS\n");
                    Main.fillArray(true); // true = randomBoard
                        try {
                            first = new Node(randomBoard[ran.nextInt(Main.chessBoardAmount)]);
                        }
                        catch (Exception bde) {
                            System.out.print("\n\n bad dimension\n");
                            return;
                        }
                        problemSolution(first);
            }
            if (args[0].equals("-d")) {
                System.out.println("\nDIAGONAL CHESSBOARD\n");
                Main.fillArray(false); // false = diagonalBoard
                try {
                    first = new Node(Main.diagonalBoard);
                }
                catch (Exception bde) {
                    System.out.print("\n\n bad dimension\n");
                    return;
                }
                problemSolution(first);
            }
        }
        
        if (args.length == 0) {
            System.out.println("\nDEFAULT CHESSBOARD\n");
            try {
                first = new Node(Main.defaultBoard);
            }
            catch (Exception bde) {
                System.out.print("\n\n bad dimension\n");
                return;
            }            
            problemSolution(first);
        }        
    }
}
