/*
 * Node.java
 *
 * Created on 13. listopad 2006, 15:00
 *
 * @author Jiri Kremser
 * @version 1.0
 */

package dam;
    import java.util.*;

/**
 * class Main
 */
public class Node {    
    public static int N = 8;
    private int[] position = new int[N];

        
    /**
     * Constructor  <Node (int[], Node)>
     *
     * @param position chessboard
     * @param prev previous node 
     */
    public Node(int[] position) {
        for (int i = 0; i < Node.N; i++) {
            this.position[i] = position[i];
        }        
    }    
    
    /**
     * Constructor 2  <Node (Node, Node)>
     *
     * @param node actual node
     * @param prev previous node
     */
    public Node(Node node) {        
        this.position = node.position;
    }        
        
    /**
     * int getC(int,int)
     *
     * @param x x-coordinate
     * @param y y-coordinate
     * @return number of queen-conflicts at the particular place
     */
    private int getC(int x,int y) {
        int c = 0;

        for (int i = 0 ; i < Node.N; i++) {
            if (this.position[i] == y) {
                c++;
            }        
            if (i != Node.N - 1) { // ! last loop
                if ( ((x + i < Node.N) && (this.position[x + i] == (y + 1 + i)))
                || ((x + i < Node.N) && (this.position[x + i] == (y - 1 - i)))
                || ((x - 2 - i >= 0) && (this.position[x - i - 2] == (y - 1 - i)))
                || ((x - 2 - i >= 0) && (this.position[x - i - 2] == (y + 1 + i))) ){
                    c++;
                }
            }
        }
        return c;
    }
                       
    /**
     * int getMaxQueenC()
     *
     * @return ordinary number of most conflict queens
     */
    private int getCQueen() {
        class Queen implements Comparable<Queen> {
            private int id;
            private int conflicts;     
            
            Queen(int id, int conflicts) {
                this.id = id;
                this.conflicts = conflicts;
            }                 
            public int compareTo(Queen queen) { return (this.conflicts - queen.conflicts); }            
        };
        
        ArrayList<Queen> cQueens = new ArrayList<Queen>();
        for (int i = 0; i < Node.N; i++) cQueens.add(new Queen(i+1,getC(i+1,position[i]) - 1));
        Collections.shuffle(cQueens,new Random());
        return Collections.max(cQueens).id;
    }    
    
    /**
     * int getMinCmoves(int)
     *
     * @param queen number of the queen
     * @return y-coordinates of minimal conflict moves of the particular queen
     */
    private int getMinCmove(int queen) {
        class cMove implements Comparable<cMove>{
            private int conflicts;
            private int where;

            cMove(int conflicts,int where) {
                this.conflicts = conflicts;
                this.where = where;
            }                 
            public int compareTo(cMove cmove) { return (this.conflicts - cmove.conflicts); }
        };
        
        List<cMove> cMoves = new ArrayList<cMove>();
        for (int i = 0; i < Node.N ; i++) cMoves.add(new cMove(this.getC(queen + 1,i + 1),i + 1));
        Collections.shuffle(cMoves,new Random());
        return Collections.min(cMoves).where;
    }
    
    /**
     * Node move(int,int)
     *
     * @param queen number of the queen
     * @param HPosition y-coordinate (destination)
     * @return result of movement (new node)
     */
    private Node move(int queen, int HPosition) {
        Node nod = new Node(this.position);
        nod.position[queen - 1] = HPosition;
        return nod;
    }
    
    /**
     * Node selectBest()
     * 
     * Selects one of the best moves
     * 
     * @return one of the best moves
     * @throws java.lang.Exception 
     */
    public Node selectBest() {  
        Set<Integer> conflicts = new TreeSet<Integer>();
        int cQueen = this.getCQueen();
        int kam = this.getMinCmove(cQueen);
        return move(cQueen,kam);
    }                                                
    
    /**
     * boolean isEnd()
     *
     * @return true if the problem has been solved
     */
    public boolean isEnd() {
        int sum = 0;
        for (int i = 0; i < Node.N; i++) {
            sum += getC(i+1,position[i]) - 1;
        }
        return (sum == 0) ? true : false;
    }
        
    /**
     * String toString()
     */    
    public String toString() {
        String board = "   ---------------------";
        for (int h = 0; h < Node.N - 5; h++) {
            board += "----";
        }
        board += "\n";
        for (int i = Node.N; i > 0; i--) {
            for (int j = 0; j < Node.N; j++) {
                if (j == 0) {
                    board += i + " " + ((i >= 10)?"":" ");
                }
                if (position[j] == i) {
                    board += "| Q ";
                }
                else board += "|   ";                
            }
            board += "|\n   ---------------------";
            for (int j = 0; j < Node.N - 5; j++) {
                board += "----";
            }
            board += "\n";
        }
        board += "     A   B   C   D   E";
        for (int h = 0; h < Node.N - 5; h++) {
            board += "   " + (char) ((int) 'F' + h) ;
        }
        
        return board;
    }
}